<?php
namespace App\Controller;

use App\Exception\SqlException as SqlException;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class WebhookController {  

    private $connection;
    private $webhookService;
    private $util;

    public function __construct(\Doctrine\DBAL\Connection $connection,
                                \App\Service\WebhookService $webhookService,
                                \App\Util\Util $util) {

        $this->connection       = $connection;
        $this->webhookService   = $webhookService;
        $this->util             = $util;
    }

    public function welcome(Request $request, Response $response) {
        return $response->withJson([
            'success' => true,
            'message' => "It works",
            'connection' => $this->connection->ping(),
            'data'    => []
        ]);
    }

    public function order(Request $request, Response $response) {
        $response->withStatus(200);
        
        try {
            $input = file_get_contents('php://input');
        
            // $this->debugLog($request->getHeaderLine('Authorization'));
            // $headers = print_r($request->getHeaders(), true); 
            // $this->debugLog($headers);

            if(empty($input)) 
                throw new \Exception("Não foram encontrados parâmetros de entrada!");
            
            $this->util->writeLog('events', (date("d-m-Y H-i-s") . ' ' . time() . '.txt'), $input);
            
            $body  = json_decode($input, true);
            $token = $request->getHeaderLine('Authorization'); //$body['meta']['user_id'];
            
            // $auth  = $this->webhookService->getAuthTokens();
            $clientData = $this->webhookService->getClientData($token);

            $url        = $clientData['URL'];
            $clientName = $clientData['NOME'];
            $clientType = $clientData['TIPO'];
            $dbName     = $clientData['DBNAME'];

            if(empty($url))
                throw new \Exception("Não há cliente associado ao token $token!");
               
            if(!empty($body["canceled"]) && $body["canceled"] == true) 
                $url .= "/cancelOrder";
            else 
                $url .= "/order";
                
            $header = [
                "Content-type: application/json",
                "x-client-name: $clientName"
            ];
            
            if($clientType == 'hosting' || $clientType == 'saas'){
                $header = [
                    "Content-type: application/json",
                    "x-client-name: $clientName",
                    "x-client-type: $clientType",
                    "x-client-env: $dbName"
                ];
            }

            $requestContent = [
                'auth'     => '81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E',
                //'uberAuth' => $auth,
                'anotaAiData' => $body
            ];
            $requestResult = $this->util->request($url, json_encode($requestContent), $header);
            $this->util->writeLog("requests", "requests-log.txt", $requestResult);
        } catch(\Exception $e) {
            $this->util->writeLog('general', (date("d-m-Y H-i-s") . ' ' . time() . '.txt'), $e->getMessage());
        }
    }

    private function debugLog($params){
        $folder      = "/home/developer/workfolder/logs/anotaAi/debugWH";
        $fileName = round(microtime(true)*1000).'_order';
        $currentFile = "$folder/$fileName";
        if(!file_exists($folder)) {
            mkdir($folder, 0775, true);
        }
        // file_put_contents($currentFile, json_encode($params, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        file_put_contents($currentFile, $params);
    }

    //run at every monday at 9hs | Cron: 0 9 * * 1 
    public function databaseBackup(Request $request, Response $response) {
        $body = file_get_contents(__DIR__ . "/../../templates/defaultMail.html");
        
        $this->util->sendDatabase("Uber Eats Webhook", $body, [
            ":DATA_ATUAL"    => date("d/m/Y"),
            ":GERADO_EM"     => date("d/m/Y H:i:s")
        ]);

        return $response->withJson([
            'success' => true,
            'message' => "",
            'data'    => []
        ]);
    }

    public function getClientList(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->getClientList()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function addClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->addClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function editClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->editClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function deleteClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "Cliente deletado com sucesso!",
                'data'    => $this->webhookService->deleteClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function addToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->addToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function editToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->editToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function deleteToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "Token deletado com sucesso!",
                'data'    => $this->webhookService->deleteToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
     public function cleanToken(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "Token removido com sucesso!",
                'data'    => $this->webhookService->cleanToken()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    public function renovateToken(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "Token renovado com sucesso!",
                'data'    => $this->webhookService->getAuthTokens()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
}