<?php

namespace App\Util;

use App\Exception;
use App\Util\Security as Security;
 
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception as MailerException;

class Util {

    private $connection;
    private $dbSchema = 'pt-br';
    private $accessToken;

    public function __construct(\Doctrine\DBAL\Connection $connection)
    {
        $this->connection = $connection;
        $this->loadDbSchema();
    }

    private function getDbSchema()
    {
        return $this->dbSchema;
    }

    private function loadDbSchema()
    {
        // $configFile = file_get_contents(__DIR__ . "/../../environment/config.json");
        // $config     = json_decode($configFile, true);
        
        // if(isset($config["dbSchema"])){
        //     $this->dbSchema = $config["dbSchema"];
        // }
    }

    /* NOVO_CODIGO_NT */
    public function newCode($counterName, $size)
    {
        try {
            $nrseq = null;
            $connection = $this->connection;
            $storedProcedure = new StoredProcedure($connection, "NOVO_CODIGO_NT");
            $storedProcedure->addParam(new Param("P_CONTADOR", Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
            $storedProcedure->addParam(new Param("P_SEQUENCIAL", Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
            $result = $storedProcedure->execute();
            return substr($result["P_SEQUENCIAL"], strlen($result["P_SEQUENCIAL"]) - $size);
        } catch (\Exception $e) {
            throw new Exception\SqlException("Erro ao executar procedure Novo Código. Err " . $e->getMessage());
        }
    }

    /**
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array())
    {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }

        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure)
    {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }

    public function getMailList()
    {
        $list = [];

        $sql = "
            SELECT DSEMAILTEC 
              FROM LISTAEMAILTEC
             WHERE CDPROJETO = '016'
        ";
        
        $result = $this->connection->fetchAll($sql);
        
        if(empty($result)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        foreach($result as $row) {
            if(!empty($row['DSEMAILTEC']) && filter_var($row['DSEMAILTEC'], FILTER_VALIDATE_EMAIL)) {
                array_push($list, $row['DSEMAILTEC']);
            }
        }
        
        if(empty($list)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        return $list;
    }

    public function writeLog($type, $fileName, $content)
    {
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";

        $basePath     = $config['logPath'] . "/$clientFolder";

        if($type == 'order') {
            $folder      = $basePath . "/orders/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_processed') {
            $folder      = $basePath . "/processed/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_error') {
            $folder      = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }
        
        if($type == 'cancelled') {
            $folder      = $basePath . "/cancelled/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'accepted') {
            $folder      = $basePath . "/accepted/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'rejected') {
            $folder      = $basePath . "/rejected/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
        
        if($type == 'cancelled_error') {
            $folder      = $basePath . "/cancelled_error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'error') {
            $folder      = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
        
        if($type == 'execution') {
            $folder      = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
            $currentFile = "$folder" . date("d") . ".txt";
            
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content, FILE_APPEND);
        }
        
        //$this->debugLog($currentFile);
    }

    // private function debugLog($params)
    // {
    //     $folder      = "/home/developer/workfolder/logs/anotaAi/debugLog";
    //     $fileName = round(microtime(true)*1000).'_order';
    //     $currentFile = "$folder/$fileName";
    //     if(!file_exists($folder)) {
    //         mkdir($folder, 0775, true);
    //     }
    //     file_put_contents($currentFile, json_encode($params, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    // }

    private function decrypt($string)
    {
        $output = false;
        $encrypt_method = "AES-256-CBC";
    
        $secret_key = 'ZEEDHI';
        $secret_iv  = 'ZEEDHI IV';
    
        $key = hash('sha256', $secret_key);
        $iv  = substr(hash('sha256', $secret_iv), 0, 16);

        $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        return $output;
    } 

    public function sendMail($recipient, $cc, $subject, $body, $params) 
    {
        try {
            $configFile = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config     = json_decode($configFile, true);
            
            if(empty($params[":NOME_SERVIDOR"])){
                $params[":NOME_SERVIDOR"] = $config["server"];
            }
        
            $mail = new PHPMailer(true);

            $mail->isSMTP();                            
            $mail->Host       = $config["host"];          
            $mail->SMTPAuth   = true;                   
            $mail->Username   = $config["username"];
            $mail->Password   = $this->decrypt($config["password"]); 
            $mail->SMTPSecure = $config["SMTPSecure"];    
            $mail->Port       = $config["port"];      
            $mail->CharSet    = "UTF-8";
            $mail->Encoding   = "base64";
            
            $mail->setFrom($config["username"], "Odhen Automação Comercial");

            if(empty($cc)) {
                $cc = ["suporte.odhen@teknisa.com"];
            }

            foreach ($recipient as $r) { $mail->addAddress($r); }
            foreach ($cc as $c) { $mail->addCC($c); }
            
            $mail->isHTML(true);                                  
            $mail->Subject = $subject;
            $mail->Body    = str_replace(array_keys($params), array_values($params), $body);
            
            $mail->send();
        } catch (Exception $e) {
            var_dump("error: ".$e->getMessage());
        }
    }

    public function executeWebService($url, $method, $header, $params) 
    {

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 15);
        curl_setopt($ch, CURLOPT_FAILONERROR, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        
        //curl_setopt($ch, CURLOPT_PROXY, "http://192.168.122.121:3128");
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($ch)) || (stripos($response, "<html>"))) {
            $fileName = $this->getCurrentDateTime() . ".txt";
            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $header,
                'url'       => $url,
                'method'    => $method,
                'params'    => $params,
                'httpCode'  => $httpCode,
                'error'     => curl_error($ch),
                'response'  => $response
            ]));
            if(strpos($url, 'anota') !== false) throw new Exception\NetworkException("Erro na comunicação com a Anota Aí.");
            
            if(strpos($url, 'google') !== false) throw new Exception\NetworkException("Erro na comunicação com a Google.");
        }
        curl_close($ch);

        return $response;
    }
    
    private function formatNetworkError($networkData) 
    {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];

        return str_replace(array_keys($templateParams), array_values($templateParams), $template);
    }    

    public function tofloat($num) 
    {
        $dotPos = strrpos($num, '.');
        $commaPos = strrpos($num, ',');
        $sep = (($dotPos > $commaPos) && $dotPos) ? $dotPos : 
            ((($commaPos > $dotPos) && $commaPos) ? $commaPos : false);
       
        if (!$sep) {
            return floatval(preg_replace("/[^0-9]/", "", $num));
        } 
    
        $tmp = floatval(
            preg_replace("/[^0-9]/", "", substr($num, 0, $sep)) . '.' .
            preg_replace("/[^0-9]/", "", substr($num, $sep+1, strlen($num)))
        );
        
        $tmp2 = str_replace(".", ",", $tmp);

        if($this->getDbSchema() == 'en-us') {
            $tmp2 = str_replace(",", ".", $tmp2);
        }
        
        return $tmp2; 
    }
    
    public function removeAcentos($texto) 
    {
        $table = array('á' => 'a', 'à' => 'a', 'ã' => 'a', 'â' => 'a',
            'é' => 'e', 'ê' => 'e', 'í' => 'i', 'ó' => 'o', 'ô' => 'o',
            'õ' => 'o', 'ú' => 'u', 'ü' => 'u', 'ç' => 'c', 'Á' => 'A',
            'À' => 'A', 'Ã' => 'A', 'Â' => 'A', 'É' => 'E', 'Ê' => 'E',
            'Í' => 'I', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ú' => 'U',
            'Ü' => 'U', 'Ç' => 'C_');
        return preg_replace("[^a-zA-Z0-9_]", "", strtr($texto, $table));
    }
    
    public function getCurrentTimezone($state) 
    {
        $timezones = [
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Sao_Paulo",    "ES" => "America/Sao_Paulo",
            "GO" => "America/Sao_Paulo",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Sao_Paulo",    "PR" => "America/Sao_Paulo",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Sao_Paulo",    "RN" => "America/Fortaleza",
            "RS" => "America/Sao_Paulo",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Sao_Paulo",
            "SE" => "America/Maceio",       "SP" => "America/Sao_Paulo",
            "TO" => "America/Araguaia",    
        ];
       
        return $timezones[$state];
    }
    
    public function setAccessToken ($accessToken) 
    {
        $this->accessToken = $accessToken;
    }
    
    public function getAccessToken ()
    {
        return $this->accessToken;
    }
    
    
    /* ----- DEPRECIADO
    public function authorizeAccess ($baseUrl, $appId, $appSecret, $appShopId) {
        
        
        $this->writeLog("execution", "", date("d/m/Y H:i:s")." Realizando nova autenticação com a 99 Food..\n");
        
        $targetUrlAuth = "https://openapi.didi-food.com/v1/auth/authtoken/refresh?app_id=$appId&app_secret=$appSecret&app_shop_id=$appShopId";
        $curlAuth = curl_init();

        curl_setopt_array($curlAuth, array(
          CURLOPT_URL => $targetUrlAuth,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => '',
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_FOLLOWLOCATION => true,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => 'GET',
          CURLOPT_SSL_VERIFYPEER => false
        ));
        
        $response_auth = curl_exec($curlAuth);
        $httpCodeAuth = curl_getinfo($curlAuth, CURLINFO_HTTP_CODE);
        $accessAuth = json_decode($response_auth, true);
        
        $fileName      = date("H-i-s") . ".txt";
        if ($accessAuth["errno"] != 0 || $httpCodeAuth != 200) {
            
            $this->writeLog("execution", "", date("d/m/Y H:i:s")." Erro durante a tentativa do Refresh Token\n");

            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => [],
                'url'       => $targetUrlAuth,
                'method'    => 'GET',
                'params'    => $params,
                'httpCode'  => $httpCodeAuth,
                'error'     => curl_error($curlAuth),
                'response'  => $response_auth
            ]));
        
        }
        
        curl_close($curlAuth);
        
        $curl = curl_init();
        
        $params = [];
        
        $targetUrl = $baseUrl . "v1/auth/authtoken/get?app_id=$appId&app_secret=$appSecret&app_shop_id=$appShopId";
        
        curl_setopt_array($curl, array(
            CURLOPT_URL => $targetUrl,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
            CURLOPT_SSL_VERIFYPEER => false
        ));
        
        $curl_response = curl_exec($curl);
        $httpCode      = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $access = json_decode($curl_response, true);
        $fileName      = date("H-i-s") . ".txt";
        
        
        if (empty($access)) {
            
            $this->writeLog("execution", "", date("d/m/Y H:i:s")." Erro durante a tentativa..\n");

            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => [],
                'url'       => $targetUrl,
                'method'    => 'GET',
                'params'    => $params,
                'httpCode'  => $httpCode,
                'error'     => curl_error($curl),
                'response'  => $curl_response
            ]));
            
            curl_close($curl);
            throw new \Exception("Não foi possível recuperar o token de acesso à API. Sem resposta do endpoint.");
        } 
        
        
        if(!isset($access["data"]["auth_token"])){
            
            $this->writeLog("execution", "", date("d/m/Y H:i:s")." Erro durante a tentativa..\n");

            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => [],
                'url'       => $targetUrl,
                'method'    => 'GET',
                'params'    => $params,
                'httpCode'  => $httpCode,
                'error'     => curl_error($curl),
                'response'  => $curl_response
            ]));
            
            curl_close($curl);
            throw new \Exception("Não foi possível recuperar o token de acesso à API. Erro de credencial.");        
        }
        
        $this->writeLog("execution", "", date("d/m/Y H:i:s")." Sucesso na autenticação..\n");        
        $this->setAccessToken($access["data"]["auth_token"]);
        curl_close($curl);
    }
    */
    
    /* ----- DEPRECIADO
    public function updateAccessToken($token, $shopId) {
        $sql = "
            UPDATE INTEGRACAOFILI
               SET AUTHTOKEN      = :AUTHTOKEN,
                   AUTHTKNDTCRIAC = :AUTHTKNDTCRIAC
             WHERE CDINTEGRACAO   = (SELECT CDINTEGRACAO FROM INTEGRACAO WHERE IDINTEGRACAO = '99F')
             AND   IDTOKEN        = :IDTOKEN
        ";
    	
    	$params = [
    	    ":AUTHTOKEN"      => $token,
            ":AUTHTKNDTCRIAC" => time(),
            ":IDTOKEN"        => $shopId
    	];
    	
    	
    	$this->connection->executeQuery($sql, $params);
    }
    */
    
    public function formatNrSeqProdCom($seq)
    {
        return str_pad((string) ((int) $seq), 3, '0', STR_PAD_LEFT);
    }
    
    public function getCurrentDateTime()
    {
    	return (new \DateTime())->format('Y-m-d\TH:i:s:uO');
    }    
    
    public function getExecutionLog()
    {
        $configFile = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config = json_decode($configFile, true);
        
        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        
        $basePath     = $config['logPath'] . "/$clientFolder";
        $folder       = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
        $currentFile  = "$folder" . date("d") . ".txt";
        
        $content = "";
        
        if(!file_exists($folder)) {
            mkdir($folder, 0775, true);
            
        }
        if(!file_exists($currentFile)) {
            $content = date("d/m/Y H:i:s") . ": [    Integridade    ] Verificação de integridade.\n";
        }
        file_put_contents($currentFile, $content, FILE_APPEND);
        
        return $currentFile;
    }
    
}